#!/usr/bin/env python

# Deduplicate a fastq file

# Instructions: Find all the TODOs and complete them.

import sys
import fileinput
from Bio import SeqIO

# In case there wase no stdin and no files passed we'll
# print a helpful message and exit early.
if len(sys.argv) == 1 and sys.stdin.isatty():
  print """usage:
    cat test.fastq | fastq_dedup.py
  or
    fastq_dedup.py test.fastq
  """
  sys.exit()


# Dictionary used to dedup.
#    sequence.id => [quality_score, full_sequence]
best = {}

# read in the sequences
# build a dictionary by sequence_id, of lists containing the score and full SeqRecord.
for seq in SeqIO.parse(fileinput.input(), "fastq"):
  list_of_qualities = seq.letter_annotations['phred_quality']
  quality_score = sum(list_of_qualities)
  if seq.id in best.keys():
    if quality_score > best[seq.id][0]:
      best[seq.id] = [quality_score, seq]
  else:
    best[seq.id] = [quality_score, seq]

# output the unique fastq formatted sequences.
for item in best.values():
  quality_score, seq = item
  print seq.format("fastq")

