#!/usr/bin/env python

# std. reverse complements of a fasta file
# usage: 
#   ./dna_complement_reverse.py test.fasta
# or
#   cat test.fasta | ./dna_complement_reverse.py

import fileinput

# reverse complement of a sequence
def reverse_complement(sequence):
    reverse = { 'A':'T', 'C':'G', 'G':'C', 'T':'A' }
    complement = ""
    for letter in sequence:
        complement += reverse[letter]
    return complement[::-1]

# define a method to print a valid fasta
# representation of an info,sequence pair.
def print_complement_sequence(info,sequence):
    if info != "":
        print info
        reverse_sequence = reverse_complement(sequence)
        line_length = 80
        for i in range(0,len(reverse_sequence),line_length):
            print reverse_sequence[i:i+79]


# define variables to temporarily hold our
# info line and sequence while we concatenate
# the full sequence together.
info = ""
sequence = ""
for line in fileinput.input():
    line = line.rstrip('\n')
    if line: # skip empty lines
        if line[0] == ">": # info lines

            # Q: Why do we do this here and not at the end of the loop?
            # A: Only here do we know that we're at a sequence boundary.
            #    At the end of the loop, the next line could be more sequence data.
            print_complement_sequence(info,sequence)

            info = line
            sequence = "" # Q: Why is it important that we reset the sequence?
                          # A: In a fasta file with multiple sequences, the first one would work fine.
                          #    but each subsequent sequence would be added to the original... not what we want at all!
        else: # reverse complement current line
          sequence += line

# Q: Why do we do this again outside the loop?
# A: Otherwise we'd never print out the last sequence.
print_complement_sequence(info,sequence)
